#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>

int main(int argc, char *argv[]) {

  int startPieces = 32;
  int fields = 16;
  int playboardsToCreate = 10;

  if (argc > 1) {
    for (int i = 1; i < argc; ++i) {
      char *data[255];
      if (argv[i][0] == 'p') {
        startPieces = strtol(argv[i]+1, data, 10);
      } else if (argv[i][0] == 'b') {
        playboardsToCreate = strtol(argv[i]+1, data, 10);
      } else if (argv[i][0] == 'f') {
        fields = strtol(argv[i]+1, data, 10);
      } else {
        printf("[ERROR] Unknown command: '%s'\nUse the switches 'p', 'b' or 'f' in combinations.\n\nLike\n'p12': 12 Game pieces\n'b10': 10 Playboards\n'f16': 16 fields.\n\n", argv[i]);
        return 1;
      }
    }
  }

  if (fields % 2 != 0) {
    printf("[ERROR] Fields dont align by 2.\n");
    return 1;
  } else if (fields < 4) {
    printf("[ERROR] Cant create playboards with less than 4 fields.\n");
    return 1;
  } else if (fields > 32) {
    printf("[ERROR] Dont want to create playboards with more than 32 fields. Buy the full edition to continue!\n");
    return 1;
  }

  if (startPieces < 4) {
    printf("[ERROR] Need at least 4 gaming pieces to operate\n");
    return 1;
  } else if (startPieces > fields * 3) {
    printf("[ERROR] Too many gaming pieces, cannot be more than 3 times the fields.\n");
    return 1;
  }

  if (playboardsToCreate < 0) {
    printf("[ERROR] Need at one playboard to generate, canceling.\n");
    return 1;
  } else if (playboardsToCreate > 100) {
    printf("[ERROR] Too many playboards, in shareware, I generate only 100 a time! We are sorry.\n");
    return 1;
  }


  // Generate the playboards
  FILE *playboardStore = fopen("playboard.csv", "w");
  if (playboardStore == NULL) {
    printf("[ERROR] Cannot open 'playboard.csv' for writing playboard data.\n");
    return 1;
  }

  int generatedPlayboards = 0;
  char data[128];

  int rows = fields / 2;

  int player = 0;
  int piecesToSet = 0;
  int leftOverPieces = 0;

  int playboard[2][fields];
  int pieces[2];

  while (playboardsToCreate--) {
    srand(time(NULL) + clock());
    generatedPlayboards++;

    pieces[0] = startPieces;
    pieces[1] = startPieces;

    // Generate the playBoard
    printf("\nPlayboard #%d\n\n", generatedPlayboards);

    for (int i = 0; i < 4; ++i) {
      if (i < 2) {
        player = 0;
      } else {
        player = 1;
      }

      for (int j = 0; j < rows; ++j) {
        leftOverPieces = pieces[player];

        if (leftOverPieces > 4) {
            srand(time(NULL) + clock());
            piecesToSet = (rand() % (int) ceil(leftOverPieces / floor((rand() % 4) + 3)) );
            if (piecesToSet > leftOverPieces) {
              piecesToSet = floor(leftOverPieces-(leftOverPieces/2));
            }
        } else {
            piecesToSet = leftOverPieces;
        }

        if ((i == 1 || i == 3) && j+1 == rows && (pieces[player] - piecesToSet) != 0) {
          piecesToSet = pieces[player];
          playboard[player][j] = piecesToSet;
        } else {
          playboard[player][j] = piecesToSet;
        }

        pieces[player] -= piecesToSet;

        if ((j+1) < rows) {
          printf("%d;", piecesToSet);
          sprintf(data, "%d;", playboard[player][j]);
          fputs(data, playboardStore);
        } else {
          printf("%d", piecesToSet);
          sprintf(data, "%d", playboard[player][j]);
          fputs(data, playboardStore);
        }
      }

      printf("\n");
      fputc('\n', playboardStore);

      if (i == 1 || i == 3) {
        printf("Pieces left: %d left of %d\n\n", pieces[player], startPieces);
      }
    }



    fputc('\n', playboardStore);
  }

  fclose(playboardStore);
}

